
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/


#include "H5HL.h"

#include <stdlib.h>


/*-------------------------------------------------------------------------
 * Table API test
 *
 * Functions tested:
 *
 * H5TBmake_table 
 * H5TBread_fields_name
 * H5TBwrite_fields_name
 *
 *-------------------------------------------------------------------------
 */



#define NFIELDS  5
#define NRECORDS 8

int main( void )
{
 
 typedef struct Particle 
 {
  char   name[16];
  int    lati;
  int    longi;
  float  pressure;
  double temperature; 
 } Particle;

 /* Define a subset of Particle, with latitude and longitude fields */
 typedef struct Position 
 {
  int    lati;
  int    longi;
 } Position;

 /* Define a subset of Particle, with name and pressure fields */
 typedef struct NamePressure 
 {
  char   name[16];
  float  pressure;
 } NamePressure;
 
 /* Calculate the type_size and the offsets of our struct members */
 size_t type_size =  sizeof( Particle );
 size_t field_offset[NFIELDS] = { HOFFSET( Particle, name ),
                                  HOFFSET( Particle, lati ),
                                  HOFFSET( Particle, longi ),
                                  HOFFSET( Particle, pressure ),
                                  HOFFSET( Particle, temperature )};

 size_t field_offset_pos[2] = { HOFFSET( Position, lati ),
                                HOFFSET( Position, longi )};

 size_t field_offset_namepre[2] = { HOFFSET( NamePressure, name ),
                                    HOFFSET( NamePressure, pressure )};
   
 /* Initially no data */
 Particle  *p_data = NULL;

 /* Define field information */
 const char *field_names[NFIELDS]  = 
	{ "Name","Latitude", "Longitude", "Pressure", "Temperature" };
 hid_t      field_type[NFIELDS];
 hid_t      string_type;
 hid_t      file_id;
 hsize_t    chunk_size = 10;
 int        *fill_data = NULL;
 int        compress  = 0;
 int        i;
 Particle   p_data_out[NRECORDS];
 hsize_t    start;      /* Record to start reading/writing */
 hsize_t    nrecords;   /* Number of records to read/write */

 /* Define new values for the field "Pressure"  */
 float      pressure_in [NRECORDS] = { 0.0f,1.0f,2.0f,3.0f,4.0f,5.0f,6.0f,7.0f };
 float      pressure_out [NRECORDS];

 /* Define new values for the fields "Latitude,Longitude"  */
 Position   position_in[NRECORDS] = { {0,0},
	{10,10},
	{20,20}, 
	{30,30}, 
	{40,40},
	{50,50}, 
	{60,60},    
	{70,70} };

 Position   position_out[NRECORDS];

 /* Define new values for the fields "Name,Pressure"  */
 NamePressure   namepre_in[NRECORDS] = 
	{ {"zero",0.0f},
	{"one",   1.0f}, 
 {"two",   2.0f}, 
	{"three", 3.0f}, 
	{"four",  4.0f},
	{"five",  5.0f},
	{"six",   6.0f}, 
	{"seven", 7.0f}, 
  };

 NamePressure   namepre_out[NRECORDS];

 /* Initialize the field field_type */
 string_type = H5Tcopy( H5T_C_S1 );
 H5Tset_size( string_type, 16 );
 field_type[0] = string_type;
 field_type[1] = H5T_NATIVE_INT;
 field_type[2] = H5T_NATIVE_INT;
 field_type[3] = H5T_NATIVE_FLOAT;
 field_type[4] = H5T_NATIVE_DOUBLE;

 TESTING("writing fields by name");

 
 /* Create a new file using default properties. */
 file_id = H5Fcreate( "field_names.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );


/*-------------------------------------------------------------------------
 * Create a table named "Table1"
 *-------------------------------------------------------------------------
 */

 /* Make the table */
 if ( H5TBmake_table( "Table Title", file_id, "Table1", (hsize_t)NFIELDS,(hsize_t) NRECORDS, type_size, 
                       field_names, field_offset, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;


 /* Write the pressure field starting at record 2 */
 start    = 2;      
 nrecords = 3; 
 if ( H5TBwrite_fields_name( file_id, "Table1", "Pressure", start, nrecords, 
	  sizeof( float ), 0, pressure_in  ) < 0 )
  goto out;


 /* Write the new longitude and latitude information starting at record 2 */
 start    = 2;      
 nrecords = 3; 
 if ( H5TBwrite_fields_name( file_id, "Table1", "Latitude,Longitude", start, nrecords, 
	  sizeof( Position ), field_offset_pos, position_in  ) < 0 )
  goto out;

 /* Read back the all table */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBread_fields_name( file_id, "Table1", "Name,Latitude,Longitude,Pressure,Temperature", 
                            start, nrecords, type_size, field_offset, p_data_out ) < 0 )
  goto out;

 /* Compare the extracted table with the initial values */
 for( i = 0; i < NRECORDS; i++ ) 
 {
  if ( i >= 2 && i <= 4 )
  {
    if ( p_data_out[i].lati        != position_in[i].lati ||
         p_data_out[i].longi       != position_in[i].longi ||
         p_data_out[i].pressure    != pressure_in[i] ) 
   goto out;
  }
 }
 
/*-------------------------------------------------------------------------
 * Create a table named "Table2"
 *-------------------------------------------------------------------------
 */

 /* Make another table */
 if ( H5TBmake_table( "Table Title", file_id, "Table2", (hsize_t)NFIELDS, (hsize_t)NRECORDS, type_size, 
                       field_names, field_offset, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;


/*-------------------------------------------------------------------------
 * Write and read the "Pressure" field
 *-------------------------------------------------------------------------
 */

 /* Write the pressure field to all the records */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBwrite_fields_name( file_id, "Table2", "Pressure", start, nrecords, 
	  sizeof( float ), 0, pressure_in  ) < 0 )
  goto out;
  
 /* Read the "Pressure" field */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBread_fields_name( file_id, "Table2", "Pressure", start, nrecords, 
	  sizeof(float), 0,  pressure_out ) < 0 )
  goto out;

 /* Compare the extracted table with the initial values */
 for( i = 0; i < NRECORDS; i++ ) 
 {

  if ( pressure_out[i] != pressure_in[i] ) {
   goto out;
  }
 }

 PASSED();
 
/*-------------------------------------------------------------------------
 * Write and read the "Latitude,Longitude" fields
 *-------------------------------------------------------------------------
 */

 TESTING("reading fields by name");

 /* Write the new longitude and latitude information to all the records */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBwrite_fields_name( file_id, "Table2", "Latitude,Longitude", start, nrecords, 
	  sizeof( Position ), field_offset_pos, position_in  ) < 0 )
  goto out;

 /* Read the "Latitude,Longitude" fields */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBread_fields_name( file_id, "Table2", "Latitude,Longitude", 
	  start, nrecords, sizeof(Position), field_offset_pos, position_out ) < 0 )
  goto out;

 /* Compare the extracted table with the initial values */
 for( i = 0; i < NRECORDS; i++ ) 
 {

  if ( position_out[i].lati  != position_in[i].lati ||
       position_out[i].longi != position_in[i].longi ) {
   goto out;
  }
 }

/*-------------------------------------------------------------------------
 * Write and read the "Name,Pressure" fields
 *-------------------------------------------------------------------------
 */

 /* Write the new name and pressure information to all the records */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBwrite_fields_name( file_id, "Table2", "Name,Pressure", start, nrecords, 
	  sizeof( NamePressure ), field_offset_namepre, namepre_in  ) < 0 )
  goto out;

 /* Read the "Name,Pressure" fields */
 start    = 0;      
 nrecords = NRECORDS; 
 if ( H5TBread_fields_name( file_id, "Table2", "Name,Pressure", 
	  start, nrecords, sizeof(NamePressure), field_offset_namepre, namepre_out ) < 0 )
  goto out;

 /* Compare the extracted table with the initial values */
 for( i = 0; i < NRECORDS; i++ ) 
 {

  if ( ( strcmp( namepre_out[i].name,  namepre_in[i].name ) != 0 ) ||
         namepre_out[i].pressure != namepre_in[i].pressure ) {
   goto out;
  }
 }

    
/*-------------------------------------------------------------------------
 * end
 *-------------------------------------------------------------------------
 */
 
 /* Close the file. */
 H5Fclose( file_id );
 PASSED();

 return 0;

out:
 /* Close the file. */
 H5Fclose( file_id );
 H5_FAILED();
 return 1;

}

