
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/




#include "H5HL.h"
#include <stdlib.h>
#include <assert.h>

#define TITLE "Title"

/*-------------------------------------------------------------------------
 * Table API test
 *-------------------------------------------------------------------------
 */

int test_delete(hid_t fid, const char* name);
int test_append(hid_t fid, const char* name);
int compare_deleted(hsize_t rrecords, hsize_t dstart, hsize_t drecords, 
                    int *wbuf, int *rbuf);


int main(void)
{
 /* Identifiers */
 hid_t fid;
 int   nerrors=0;
 
 /* Create a new file using default properties. */
 fid=H5Fcreate("test_table.h5",H5F_ACC_TRUNC,H5P_DEFAULT,H5P_DEFAULT);

 TESTING("delete records");
 nerrors+=test_delete(fid, "one");
 if(nerrors>0) H5_FAILED() else PASSED();
 TESTING("append records");
 nerrors+=test_append(fid, "two");
 if(nerrors>0) H5_FAILED() else PASSED();

 assert((H5Fclose(fid)>=0));
 return 0;

}

/*-------------------------------------------------------------------------
 * test_delete
 *-------------------------------------------------------------------------
 */

int test_delete(hid_t fid, const char *name)
{
 /* make a table with one integer field */
 hsize_t nfields=1;      
 hsize_t nrecords=8;  
 hsize_t chunk_size=10;
 int     compress=0;
 int     *fill=NULL;
 hid_t   field_type[1];
 size_t  type_size=sizeof(int);
 size_t  field_offset[1]={0};
 const char *field_names[1]={"int"};
 size_t field_size[1]={sizeof(int)};
 /* write, read, delete some records */
 hsize_t rstart;
 hsize_t dstart;
 hsize_t rrecords;
 hsize_t drecords;
 hsize_t rfields;
 int     wbuf[8]={0,1,2,3,4,5,6,7};
 int     rbuf[8];
 int     nerrors=0;
 hsize_t i;

/*-------------------------------------------------------------------------
 * initialize
 *-------------------------------------------------------------------------
 */

 field_type[0]=H5T_NATIVE_INT;

/*-------------------------------------------------------------------------
 * Create a table
 * pos = 0 1 2 3 4 5 6 7  
 * data= 0 1 2 3 4 5 6 7 
 *-------------------------------------------------------------------------
 */
 if (H5TBmake_table(TITLE,fid,name,nfields,nrecords,type_size,
                    field_names,field_offset,field_type,
                    chunk_size,fill,compress,wbuf)<0) 
                    return 1;
 
 if (H5TBmake_table(TITLE,fid,"org_one",nfields,nrecords,type_size,
                    field_names,field_offset,field_type,
                    chunk_size,fill,compress,wbuf)<0) 
                    return 1;

/*-------------------------------------------------------------------------
 * Delete records, start at 2, delete 3
 * pos = 0 1 2 3 4 
 * data= 0 1 5 6 7  
 *-------------------------------------------------------------------------
 */
 dstart=2; drecords=3;
 if (H5TBdelete_record(fid,name,dstart,drecords)<0) 
  return 1;

 if (H5TBget_table_info(fid,name,&rfields,&rrecords)<0) 
  return 1;

 rstart=0; 
 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 assert(rrecords == nrecords-drecords);
 nerrors+=compare_deleted(rrecords,dstart,drecords,wbuf,rbuf);
 
/*-------------------------------------------------------------------------
 * reset compare buffer
 *-------------------------------------------------------------------------
 */
 nrecords=rrecords; 
 for( i=0; i<nrecords; i++) 
  wbuf[i] = rbuf[i];

/*-------------------------------------------------------------------------
 * Delete records, start at 0, delete 2
 * pos = 0 1 2  
 * data= 5 6 7  
 *-------------------------------------------------------------------------
 */
 dstart=0; drecords=2;
 if (H5TBdelete_record(fid,name,dstart,drecords)<0) 
  return 1;

 if (H5TBget_table_info(fid,name,&rfields,&rrecords)<0) 
  return 1;

 rstart=0; 
 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 assert(rrecords == nrecords-drecords);
 nerrors+=compare_deleted(rrecords,dstart,drecords,wbuf,rbuf);


/*-------------------------------------------------------------------------
 * reset compare buffer
 *-------------------------------------------------------------------------
 */
 nrecords=rrecords; 
 for( i=0; i<nrecords; i++) 
  wbuf[i] = rbuf[i];

/*-------------------------------------------------------------------------
 * Delete records, start at 1, delete 1
 * pos = 0 1   
 * data= 5 7 
 *-------------------------------------------------------------------------
 */
 dstart=1; drecords=1;
 if (H5TBdelete_record(fid,name,dstart,drecords)<0) 
  return 1;

 if (H5TBget_table_info(fid,name,&rfields,&rrecords)<0) 
  return 1;

 rstart=0; 
 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 assert(rrecords == nrecords-drecords);
 nerrors+=compare_deleted(rrecords,dstart,drecords,wbuf,rbuf);


/*-------------------------------------------------------------------------
 * reset compare buffer
 *-------------------------------------------------------------------------
 */
 nrecords=rrecords; 
 for( i=0; i<nrecords; i++) 
  wbuf[i] = rbuf[i];

/*-------------------------------------------------------------------------
 * Delete records, start at 0, delete 1
 * pos = 0    
 * data= 7 
 *-------------------------------------------------------------------------
 */
 dstart=0; drecords=1;
 if (H5TBdelete_record(fid,name,dstart,drecords)<0) 
  return 1;

 if (H5TBget_table_info(fid,name,&rfields,&rrecords)<0) 
  return 1;

 rstart=0; 
 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 assert(rrecords == nrecords-drecords);
 nerrors+=compare_deleted(rrecords,dstart,drecords,wbuf,rbuf);

/*-------------------------------------------------------------------------
 * reset compare buffer
 *-------------------------------------------------------------------------
 */
 nrecords=rrecords; 
 for( i=0; i<nrecords; i++) 
  wbuf[i] = rbuf[i];

 /* Read complete table */
 if (H5TBread_table(fid,name,type_size,field_offset,field_size,rbuf)<0) 
  return 1;

 /* Compare */
 for( i=0; i<rrecords; i++) 
 {
  if ( rbuf[i] != wbuf[i] ) 
  {
   printf("error in reading table");
   return 1;
  }
 }


 return nerrors;
}


/*-------------------------------------------------------------------------
 * test_append
 *-------------------------------------------------------------------------
 */


int test_append(hid_t fid, const char *name)
{
/* make a table with one integer field */
 hsize_t nfields=1;      
 hsize_t nrecords=8;  
 hsize_t chunk_size=10;
 int     compress=0;
 int     *fill=NULL;
 hid_t   field_type[1];
 size_t  type_size=sizeof(int);
 size_t  field_offset[1]={0};
 const char *field_names[1]={"int"};
 /* write, read, append, insert some records */
 hsize_t wstart;
 hsize_t rstart;
 hsize_t istart;
 hsize_t rrecords;
 hsize_t wrecords;
 hsize_t arecords;
 hsize_t irecords;
 hsize_t rfields;
 int     wbuf[12]={0,1,2,3,4,5,6,7,8,9,10,11};
 int     abuf[2]={8,9};
 int     ibuf[2]={0,0};
 int     rbuf[12];
 int     nerrors=0;
 hsize_t i, j;

/*-------------------------------------------------------------------------
 * initialize
 *-------------------------------------------------------------------------
 */

 field_type[0]=H5T_NATIVE_INT;

/*-------------------------------------------------------------------------
 * Create an empty table
 * pos = 0 1 2 3 4 5 6 7  
 * data= 0 0 0 0 0 0 0 0 
 *-------------------------------------------------------------------------
 */

 if (H5TBmake_table(TITLE,fid,name,nfields,nrecords,type_size,
                    field_names,field_offset,field_type,
                    chunk_size,fill,compress,0)<0) 
                    return 1;

/*-------------------------------------------------------------------------
 * Write records, start at 0, write 8
 * pos = 0 1 2 3 4 5 6 7  
 * data= 0 1 2 3 4 5 6 7 
 *-------------------------------------------------------------------------
 */
 wstart=0; wrecords=8;
 if (H5TBwrite_records(fid,name,wstart,wrecords,type_size,field_offset,wbuf)<0) 
  return 1;

/*-------------------------------------------------------------------------
 * Read records, start at 0, read 8
 * pos = 0 1 2 3 4 5 6 7  
 * data= 0 1 2 3 4 5 6 7
 *-------------------------------------------------------------------------
 */
 rstart=0; rrecords=8;
 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 for( i=rstart; i<rrecords; i++) 
 {
  if ( rbuf[i] != wbuf[i] ) return 1;
 }


/*-------------------------------------------------------------------------
 * Append 2 records
 * pos = 0 1 2 3 4 5 6 7 8 9 
 * data= 0 1 2 3 4 5 6 7 8 9
 *-------------------------------------------------------------------------
 */
 arecords=2;
 if (H5TBappend_records(fid,name,arecords,type_size,field_offset,abuf)<0) 
  return 1;

 if (H5TBget_table_info(fid,name,&rfields,&rrecords)<0) 
  return 1;

 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 for( i=rstart; i<rrecords; i++) 
 {
  if ( rbuf[i] != wbuf[i] ) return 1;
 }


/*-------------------------------------------------------------------------
 * Insert 2 records
 * pos = 0 1 2 3 4 5 6 7 8 9 10 11
 * data= 0 0 0 1 2 3 4 5 6 7 8  9 
 *-------------------------------------------------------------------------
 */
 istart=1; irecords=2;
 if (H5TBinsert_record(fid,name,istart,irecords,type_size,field_offset,ibuf)<0) 
  return 1;

 if (H5TBget_table_info(fid,name,&rfields,&rrecords)<0) 
  return 1;

 if (H5TBread_records(fid,name,rstart,rrecords,type_size,field_offset,rbuf)<0) 
  return 1;

 /* Compare */
 for( i=0; i<rrecords; i++) 
 {
  if (i<istart)
  {
   if ( rbuf[i] != wbuf[i] ) 
   {
    printf("error in reading table");
    return 1;
   }
  }
  else if (i>=istart && i<istart+irecords)
  {
   j=i-istart;
   if ( rbuf[i] != ibuf[j] ) 
   {
    printf("error in reading table");
    return 1;
   }
  }
  else 
  {
   j=i-irecords;
   if ( rbuf[i] != wbuf[j] ) 
   {
    printf("error in reading table");
    return 1;
   }
  }
  
 }

 return 0;
}





/*-------------------------------------------------------------------------
 * compare_deleted
 *-------------------------------------------------------------------------
 */

int compare_deleted(hsize_t rrecords, hsize_t dstart, hsize_t drecords, 
                    int *wbuf, int *rbuf)
{
 hsize_t i,j;

 for( i=0; i<rrecords; i++) 
 {
  if (i<dstart)
  {
   if ( rbuf[i] != wbuf[i] ) 
   {
    printf("error in reading table");
    return 1;
   }
  }
  else
  {
   j=i+drecords;
   if ( rbuf[i] != wbuf[j] ) 
   {
    printf("error in reading table");
    return 1;
   }
  }
  
 }
 
 return 0;
}


